package tech.nevets.autoupdater;

import com.github.dockerjava.api.DockerClient;
import com.github.dockerjava.api.model.Container;
import com.github.dockerjava.core.DefaultDockerClientConfig;
import com.github.dockerjava.core.DockerClientBuilder;
import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Arrays;

public class UpdateManager {
    public static boolean update(String jarFileName, String dockerContainerName, String workingDirectory, String downloadUrl) {
        File workingDir = new File(workingDirectory);
        if (!workingDir.isDirectory()) {
            if (Main.isVerbose) System.out.println("Is not a directory");
            return false;
        }

        DockerClient dockerClient = getDockerClient();

        // Stop container here
        dockerClient.stopContainerCmd(getContainerByName(dockerClient, dockerContainerName)).exec();

        // Delete old jar file
        File jarFile = new File(workingDirectory + "/" + jarFileName);
        if (jarFile.exists() && jarFile.isFile()) {
            if (Main.isVerbose) System.out.println("Jarfile deleted");
            return jarFile.delete();
        }

        // Download new jar file
        try {
            FileUtils.copyURLToFile(new URL(downloadUrl), jarFile, 30000, 30000);
            if (Main.isVerbose) System.out.println("Downloaded updated jarfile");
        } catch (IOException e) {
            e.printStackTrace();
            if (Main.isVerbose) System.out.println("error copying");
            return false;
        }

        // Start container here
        dockerClient.startContainerCmd(getContainerByName(dockerClient, dockerContainerName));

        return true;
    }

    private static DockerClient getDockerClient() {
        DefaultDockerClientConfig config
                = DefaultDockerClientConfig.createDefaultConfigBuilder()
                .withDockerTlsVerify("1")
                .withDockerHost(Main.CONFIG.getConfig().get("docker-socket").getAsString()).build();

        return DockerClientBuilder.getInstance(config).build();
    }

    private static String getContainerByName(DockerClient dockerClient, String containerName) {
        for (Container container : dockerClient.listContainersCmd().exec()) {
            if (Arrays.asList(container.getNames()).contains(containerName)) {
                if (Main.isVerbose) System.out.println("Container id: " + container.getId());
                return container.getId();
            }
        }
        return "";
    }
}
